/*
    Copyright (C) 1999 by Joe Pasqua <joe@NoRoomAtTheInn.org>.
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions
    are met:
    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.


    THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;

class XAControlType extends Type {

    XAControlType() {
        methods = new Hashtable(1);
        methods.put("UserDefinedMethod", new UserDefinedMethod());
    }

    static class UserDefinedMethod implements Type.Method {

        public short[] codeGen(Value target, Vector inputs)
                throws ParseException
        {
            // OK, the first input argument is actually the name
            // of the user defined method. The rest of the inputs are
            // the arguments.
            String  methodName = (String)inputs.elementAt(0);

            UserMethod  method =
                    (UserMethod)XACompiler.userMethods.get(methodName);
            if (method == null)
                throw new ParseException("Undefined method: " + methodName);

            if (inputs.size() != method.args.size()+1)
                throw new ParseException(
                    "Wrong number of arguments: found " + (inputs.size()-1) +
                    ", expecting " + method.args.size());

            XACompiler.symTab.push();
            try {
                for (int i = 0; i < method.args.size(); i++) {
                    BasicVal    input = (BasicVal)inputs.elementAt(i+1);
                    Arg         arg = (Arg)method.args.elementAt(i);

                    if (input.isLiteral) {
                        XACompiler.symTab.addConst(
                                arg.getName(), arg.getTypeName(), input.value);
                    }
                    else {
                        Symbol  s = XACompiler.symTab.get(input.value);
                        if (s == null)
                            throw new ParseException(
                                    "Undefined variable: " + input.value);
                        if (s.isVar) {
                            XACompiler.symTab.put(arg.getName(), s);
                        }
                        else {
                            // TO DO: The following should be a call on
                            // asBasicVal() instead of toString().
                            String  value = s.initVal.toString();
                            XACompiler.symTab.addConst(
                                arg.getName(), arg.getTypeName(), value);
                        }
                    }
                }

                for (int i = 0; i < method.statements.size(); i++) {
                    MethodCall  m = (MethodCall)method.statements.elementAt(i);
                    // TO DO: The bias parameter needs to be passed in to
                    // this method so we can pass it in to the call on codeGen
                    // and the call on genComment
                    short[] code = XACompiler.codeGen(XACompiler.bias, m);
                    if (code != null) {
                        String  comment =
                            XACompiler.genComment(
                                XACompiler.bias == 0 ? "Then" : "Else",
                                m);
                        XACompiler.emit(code, comment, XACompiler.fos);
                    }
                }
            } finally {
                XACompiler.symTab.pop();
            }
            return null;
        }

    }
}

